﻿using SciChart.Charting.Visuals;
using SciChartPerfTester.ViewModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace SciChartPerfTester.Control
{
	/// <summary>
	/// Interaction logic for ChartControl.xaml
	/// </summary>
	public partial class ChartControl : UserControl
	{
		public ChartControl()
		{
			InitializeComponent();

		}

		private void UserControl_DataContextChanged(object sender, DependencyPropertyChangedEventArgs e)
		{
			// HACK to try to use SuspendUpdate on the Surface itself
			var vm = e.NewValue as ChartViewModel;
			vm.SciChartSurface = sciChartSurface;

		}

		private static object _FpsSyncLock = new object();
		private static bool _fpsCreated = false;
		private void UserControl_Loaded(object sender, RoutedEventArgs e)
		{
			if (!_fpsCreated)
			{
				lock (_FpsSyncLock)
				{
					if (!_fpsCreated)
					{
						// Assure that only the first chart will have events wired up for minimal impact
						CompositionTarget.Rendering += CompositionTarget_Rendering;
						sciChartSurface.Rendered += SciChartSurface_Rendered;
						_fpsCreated = true;
					}
				}
			}
		}

		private int _crFrameCount = 0;
		private DateTime _crLastUpdateTime;
		private void CompositionTarget_Rendering(object sender, EventArgs e)
		{
			_crFrameCount++;
		}

		private int _sciChartFrameCount = 0;
		private DateTime _sciChartLastUpdateTime;
		private void SciChartSurface_Rendered(object sender, EventArgs e)
		{
			_sciChartFrameCount++;
			var now = DateTime.Now;
			var timeSinceLastUpdate = now.Subtract(_sciChartLastUpdateTime);
			if (timeSinceLastUpdate > TimeSpan.FromSeconds(1))
			{
				FpsTextBlock.Text = $"SC.FPS: {(_sciChartFrameCount / timeSinceLastUpdate.TotalSeconds).ToString("000.000") }";
				CompositionRenderingTextBlock.Text = $"CR.FPS: {(_crFrameCount / timeSinceLastUpdate.TotalSeconds).ToString("000.000")}";

				_sciChartLastUpdateTime = now;
				_sciChartFrameCount = 0;

				_crLastUpdateTime = now;
				_crFrameCount = 0;
			}
		}
	}
}
